﻿--[[
Name: PratChannelRenumbering
Revision: $Revision: 24346 $
Author(s): Sylvanaar (sylvanaar@mindspring.com)
Website: http://www.wowace.com/files/index.php?path=Prat/
Documentation: http://www.wowace.com/wiki/Prat/Integrated_Modules#ChannelRenumbering
Subversion: http://svn.wowace.com/wowace/trunk/Prat/
Discussions: http://groups.google.com/group/wow-prat
Issues and feature requests: http://code.google.com/p/prat/issues/list
Description: Module for Prat that renumbers chat channels to your specification.
Dependencies: Prat
]]

-- set prat module name
local PRAT_MODULE = Prat:RequestModuleName("PratChannelRenumbering")

-- define localized strings
local L = AceLibrary("AceLocale-2.2"):new(PRAT_MODULE)

L:RegisterTranslations("enUS", function() return {
    ["ChannelRenumbering"] = true,
    ["Channel renumbering options."] = true,
    ["Save"] = true,
    ["Save channel list based on current channels"] = true,
    ["Sort"] = true,
    ["Sort channel list in order"] = true,
    ["Toggle"] = true,
    ["Toggle sorting channels on load on and off."] = true,
    ["Sort channel list in order"] = true,
    ["Sorting on load"] = true,
    ["Edit List"] = true,
    ["Edit current order list"] = true,
    ["%dth channel"] = true,
    ["Set %dth channel."] = true,
    ["<channel name>"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
    ["ChannelRenumbering"] = "채널번호변경",
    ["Channel renumbering options."] = "채널 번호변경 설정입니다.",
    ["Save"] = "저장",
    ["Save channel list based on current channels"] = "현재 채널을 기반으로 채널 목록을 저장합니다.",
    ["Sort"] = "정렬",
    ["Sort channel list in order"] = "채널 목록 순서를 정렬합니다.",
--    ["Toggle"] = true,
    ["Toggle sorting channels on load on and off."] = "접속 시 채널 정렬 기능을 사용합니다.",
    ["Sort channel list in order"] = "채널 목록 정렬 순서",
    ["Sorting on load"] = "로드 시 정렬",
    ["Edit List"] = "목록 편집",
    ["Edit current order list"] = "현재 순서 목록을 편집합니다.",
    ["%dth channel"] = "%d번째 채널",
    ["Set %dth channel."] = "%d번째 채널을 설정합니다.",
    ["<channel name>"] = "<채널명>",
} end)

L:RegisterTranslations("esES", function() return {
    ["ChannelRenumbering"] = "Renumeraci\195\179n de Canales",
    ["Channel renumbering options."] = "Opciones de renumeraci\195\179n de canales",
    ["Save"] = "Guardar",
    ["Save channel list based on current channels"] = "Guarda la lista de canales basada en los canales actuales",
    ["Sort"] = "Ordenar",
    ["Sort channel list in order"] = "Ordena la lista de canales",
    ["Toggle"] = "Activar",
    ["Toggle sorting channels on load on and off."] = "Determina si ordena los canales al cargar",
    ["Sort channel list in order"] = "Ordena la lista de canales",
    ["Sorting on load"] = "Ordenar al cargar",
    ["Edit List"] = "Editar lista",
    ["Edit current order list"] = "Edita la lista de orden actual",
    ["%dth channel"] = "Canal n\195\186m. %d",
    ["Set %dth channel."] = "Establecer como canal n\195\186mero %d",
    ["<channel name>"] = "<nombre de canal>",
} end)

L:RegisterTranslations("deDE", function() return {
    ["ChannelRenumbering"] = "Kanal Neu Nummerrierung",
    ["Channel renumbering options."] = "Kanal Neu Nummerrierungs Optionen",
    ["Save"] = "Speichern",
    ["Save channel list based on current channels"] = "Speichere die Kanalliste basierend auf den gegenw\195\164rtigen Kan\195\164len",
    ["Sort"] = "Sortiere",
    ["Sort channel list in order"] = "Sortiere die Kanal Liste nach Anordnung.",
    ["Toggle"] = "Lade Sortierung",
    ["Toggle sorting channels on load on and off."] = "Schaltet die Sortierung beim Laden an und aus.",
    ["Sort channel list in order"] = "Sortiere die Kanal Liste nach Anordnung.",
    ["Sorting on load"] = "Sortiere beim Laden",
    ["Edit List"] = "Editiere Liste",
    ["Edit current order list"] = "Editiere gegenw\195\164rtige Anordnungsliste",
    ["%dth channel"] = "%dter Kanal",
    ["Set %dth channel."] = "Setze %dten Kanal",
    ["<channel name>"] = "<kanal name>",
} end)

-- define bogus channels
local boguschannels = {
    [1] =  "bgrc8193",
    [2] =  "bgrc9443",
    [3] =  "bgrc4231",
    [4] =  "bgrc5423",
    [5] =  "bgrc7853",
    [6] =  "bgrc7542",
    [7] =  "bgrc9534",
    [8] =  "bgrc1546",
    [9] =  "bgrc8452",
    [10] = "bgrc2654",
}

-- get prat module categories
local cat = Prat.Categories

-- create prat module
Prat_ChannelRenumbering = Prat:NewModule(PRAT_MODULE)
Prat_ChannelRenumbering.revision = tonumber(string.sub("$Revision: 24346 $", 12, -3))

-- define key module values
Prat_ChannelRenumbering.moduleName = L["ChannelRenumbering"]
Prat_ChannelRenumbering.moduleDesc = L["Channel renumbering options."]
Prat_ChannelRenumbering.consoleName = "chanrenumbering"
Prat_ChannelRenumbering.guiName = L["ChannelRenumbering"]
Prat_ChannelRenumbering.Categories = { cat.CHANNEL }
Prat_ChannelRenumbering.status = "BROKEN"

-- define the default db values
Prat_ChannelRenumbering.defaultDB = {
        on = false,
        sortonload = false,
}

-- create a moduleOptions stub (for setting self.moduleOptions)
Prat_ChannelRenumbering.moduleOptions = {}

-- add module options not covered by templates
function Prat_ChannelRenumbering:GetModuleOptions()
    self.moduleOptions = {
        name = L["ChannelRenumbering"],
        desc = L["Channel renumbering options."],
        type = "group",
        args = {
            sort = {
                name = L["Sort"],
                desc = L["Sort channel list in order"],
                type = "execute",
                order = 100,
                func = function() self:Sort() end,
                disabled = function() return self.currentlysorting end,
            },
            save = {
                name = L["Save"],
                desc = L["Save channel list based on current channels"],
                type = "execute",
                order = 110,
                func = function() self:Save() end,
                disabled = function() return self.currentlysorting end,
            },
            edit = {
                name = L["Edit List"],
                desc = L["Edit current order list"],
                type = "group",
                disabled = function() return self.currentlysorting end,
                order = 120,
                args = {
                    channel4 = {
                        name = string.format(L["%dth channel"], 4),
                        desc = string.format(L["Set %dth channel."], 4),
                        type = "text",
                        usage = L["<channel name>"],
                        order = 121,
                        get = function() return self.db.char.orderlist[4] end,
                        set = function(v) self.db.char.orderlist[4] = v end,
                    },
                    channel5 = {
                        name = string.format(L["%dth channel"], 5),
                        desc = string.format(L["Set %dth channel."], 5),
                        type = "text",
                        order = 122,
                        usage = L["<channel name>"],
                        get = function() return self.db.char.orderlist[5] end,
                        set = function(v) self.db.char.orderlist[5] = v end,
                    },
                    channel6 = {
                        name = string.format(L["%dth channel"], 6),
                        desc = string.format(L["Set %dth channel."], 6),
                        type = "text",
                        order = 123,
                        usage = L["<channel name>"],
                        get = function() return self.db.char.orderlist[6] end,
                        set = function(v) self.db.char.orderlist[6] = v end,
                    },
                    channel7 = {
                        name = string.format(L["%dth channel"], 7),
                        desc = string.format(L["Set %dth channel."], 7),
                        type = "text",
                        order = 124,
                        usage = L["<channel name>"],
                        get = function() return self.db.char.orderlist[7] end,
                        set = function(v) self.db.char.orderlist[7] = v end,
                    },
                    channel8 = {
                        name = string.format(L["%dth channel"], 8),
                        desc = string.format(L["Set %dth channel."], 8),
                        type = "text",
                        order = 125,
                        usage = L["<channel name>"],
                        get = function() return self.db.char.orderlist[8] end,
                        set = function(v) self.db.char.orderlist[8] = v end,
                    },
                    channel9 = {
                        name = string.format(L["%dth channel"], 9),
                        desc = string.format(L["Set %dth channel."], 9),
                        type = "text",
                        order = 126,
                        usage = L["<channel name>"],
                        get = function() return self.db.char.orderlist[9] end,
                        set = function(v) self.db.char.orderlist[9] = v end,
                    },
                    channel10 = {
                        name = string.format(L["%dth channel"], 10),
                        desc = string.format(L["Set %dth channel."], 10),
                        type = "text",
                        order = 127,
                        usage = L["<channel name>"],
                        get = function() return self.db.char.orderlist[10] end,
                        set = function(v) self.db.char.orderlist[10] = v end,
                    },
                },
            },
            load = {
                name = L["Sorting on load"],
                desc = L["Toggle sorting channels on load on and off."],
                type = "toggle",
                order = 130,
                get = function() return self.db.profile.sortonload end,
                set = function(v) self.db.profile.sortonload = v end,
            }
        }
    }
    return self.moduleOptions
end

--[[------------------------------------------------
    Module Event Functions
------------------------------------------------]]--

-- things to do when the module is initialized
function Prat_ChannelRenumbering:OnModuleInit()
    Prat:RegisterDefaults("PratChannelRenumbering", "char", {
        orderlist = {},
    })
    self.currentlysorting = false
end

-- things to do when the module is enabled
function Prat_ChannelRenumbering:OnModuleEnable()
    -- register events
    if self.db.profile.sortonload then
        self:RegisterEvent("AceEvent_FullyInitialized")
    end
end

-- things to do when the module is disabled
function Prat_ChannelRenumbering:OnModuleDisable()
    -- unregister events
    self:UnregisterAllEvents()
end

--[[------------------------------------------------
    Core Functions
------------------------------------------------]]--

function Prat_ChannelRenumbering:AceEvent_FullyInitialized()
    if self.db.profile.sortonload then
        self:ScheduleEvent("Sort", self.Sort, 1, self)
    end
end

function Prat_ChannelRenumbering:Save()
    self.db.char.orderlist = {}
    for i=4,10 do -- 1-3 are allways reserved for globalcomm so lets not touch it
        local cid, cname = GetChannelName(i)
        if cid > 0 and cname ~= nil then
            self.db.char.orderlist[cid] = cname
        end
    end
end

function Prat_ChannelRenumbering:Sort()
    self.currentlysorting = true
    self:Hook("ChatFrame_MessageEventHandler", true)
    self:LeaveChannels()
    self:ScheduleEvent("JoinChannels", self.JoinChannels, 1, self)
end

function Prat_ChannelRenumbering:LeaveChannels()
    self.currentleftlist = {}
    for i=1,3 do
        local cid, cname = GetChannelName(i)
        for n,v in pairs(self.db.char.orderlist) do
            if v == cname then
                self:Debug("Leaving "..cid.." - "..cname)
                LeaveChannelByName(cname)
            end
        end
    end
    for i=4,10 do -- 1-3 are allways reserved for globalcomm so lets not touch it
        local cid, cname = GetChannelName(i)
        if cid > 0 and cname ~= nil then
            if cname ~= self.db.char.orderlist[cid] then
                self:Debug("Leaving "..cid.." - "..cname)
                LeaveChannelByName(cname)
                self.currentleftlist[cid] = cname
            end
        end
    end
end

function Prat_ChannelRenumbering:JoinChannels()
    local num = #self.currentleftlist
    local bogus = false
    for i=1,3 do
        local j, jc = GetChannelName(i)
        if j == 0 and jc == nil then
            self:Join(boguschannels[i], true)
        end
    end
    for i=4,10 do -- 1-3 are allways reserved for globalcomm so lets not touch it
        local j, jc = GetChannelName(i)
        if j == 0 and jc == nil then
            local jname = self.db.char.orderlist[i]
            if jname ~= nil and jname ~= "" then
                if num > 0 then
                    if self.currentleftlist[i] ~= nil and self.currentleftlist[i] ~= "" then
                        jname = self.currentleftlist[i]
                        self.currentleftlist[i] = nil
                        num = num - 1
                    else
                        for n,v in pairs(self.currentleftlist) do
                            if v ~= nil and v ~= "" then
                                jname = self.currentleftlist[n]
                                self.currentleftlist[n] = nil
                                num = num - 1
                            end
                        end
                    end
                end
            else
                jname = boguschannels[i]
                bogus = true
            end
            if jname then
                self:Debug("Doing Join "..i.." - "..jname)
                local cid, cname = GetChannelName(jname)
                if cid == 0 and cname == nil then
                        self:Join(jname, bogus)
--                  JoinChannelByName(jname)
--                  AddChatWindowChannel(getglobal("ChatFrame1"):GetID(), jname) -- Active in chatframe1 doesnt work before reloadui for some reason...
                end
            end
        end
    end
    self:ScheduleEvent("LeaveBogusChannels", self.LeaveBogusChannels, 1, self)
end

function Prat_ChannelRenumbering:LeaveBogusChannels()
    for k,v in pairs(boguschannels) do
        local cid, cname = GetChannelName(v)
        if cid > 0 then
            LeaveChannelByName(cid)
        end
    end
    self:ScheduleEvent("SortingDone", self.SortingDone, 1, self)
end

function Prat_ChannelRenumbering:SortingDone()
    if self:IsHooked("ChatFrame_MessageEventHandler") then
        self:Unhook("ChatFrame_MessageEventHandler")
    end
    self.currentlysorting = false
end

function Prat_ChannelRenumbering:ChatFrame_MessageEventHandler(event)
    local type = strsub(event, 10);
    if (type == "CHANNEL_NOTICE") then return true end
    return self.hooks["ChatFrame_MessageEventHandler"](event)
end

function Prat_ChannelRenumbering:Join(name, bogus)
    local zoneChannel, channelName = JoinChannelByName(name, password, DEFAULT_CHAT_FRAME:GetID())
    if channelName then
        name = channelName
    end
    if not zoneChannel then
        return
    end
    local i = 1
    while DEFAULT_CHAT_FRAME.channelList[i] do
        i = i + 1
    end
    DEFAULT_CHAT_FRAME.channelList[i] = name
    DEFAULT_CHAT_FRAME.zoneChannelList[i] = zoneChannel
end