﻿--[[
Name: PratChannelSeparator
Revision: $Revision: 24346 $
Author(s): Krtek (krtek4@gmail.com)
           Sylvanaar (sylvanaar@mindspring.com)
Website: http://www.wowace.com/files/index.php?path=Prat/
Documentation: http://www.wowace.com/wiki/Prat/Integrated_Modules#ChannelSeparator
Subversion: http://svn.wowace.com/wowace/trunk/Prat/
Discussions: http://groups.google.com/group/wow-prat
Issues and feature requests: http://code.google.com/p/prat/issues/list
Description: Module for Prat that separates chat channels in the chat setting menu (default=on).
Dependencies: Prat
]]

-- set prat module name
local PRAT_MODULE = Prat:RequestModuleName("PratChannelSeparator")

-- define localized strings
local L = AceLibrary("AceLocale-2.2"):new(PRAT_MODULE)

L:RegisterTranslations("enUS", function() return {
    ["ChannelSeparator"] = true,
    ["Separates chat channels in chat setting menu."] = true,
} end)

L:RegisterTranslations("deDE", function() return {
    ["ChannelSeparator"] = "Kanal Trennen",
    ["Separates chat channels in chat setting menu."] = "Trennt Chat Kan\195\164le in dem Chateinstellungs Men\195\188.",
} end)

L:RegisterTranslations("esES", function() return {
    ["ChannelSeparator"] = "Separador de Canal",
    ["Separates chat channels in chat setting menu."] = "Separa los canales de chat en el men\195\186 de ajustes del chat",
} end)

L:RegisterTranslations("koKR", function() return {
    ["ChannelSeparator"] = "채널구분",
    ["Separates chat channels in chat setting menu."] = "대화 설정 메뉴에 대화 채널을 구분합니다.",
} end)

L:RegisterTranslations("zhCN", function() return {
    ["ChannelSeparator"] = "频道分离",
    ["Separates chat channels in chat setting menu."] = "分离聊天频道设置菜单。",
} end)

L:RegisterTranslations("zhTW", function() return {
    ["ChannelSeparator"] = "頻道分離",
    ["Separates chat channels in chat setting menu."] = "在聊天設定選單中分離聊天頻道。",
} end)

BATTLEGROUND = TEXT(CHAT_MSG_BATTLEGROUND)

-- get prat module categories
local cat = Prat.Categories

-- create prat module
Prat_ChannelSeparator = Prat:NewModule(PRAT_MODULE)
Prat_ChannelSeparator.revision = tonumber(string.sub("$Revision: 24346 $", 12, -3))

-- define key module values
Prat_ChannelSeparator.moduleName = L["ChannelSeparator"]
Prat_ChannelSeparator.moduleDesc = L["Separates chat channels in chat setting menu."]
Prat_ChannelSeparator.consoleName = "chanseparator"
Prat_ChannelSeparator.guiName = L["ChannelSeparator"]
Prat_ChannelSeparator.Categories = { cat.CHANNEL }
Prat_ChannelSeparator.status = "TAINT"

-- define the default db values
Prat_ChannelSeparator.defaultDB = {
    on = true,
}

-- create a moduleOptions stub (for setting self.moduleOptions)
Prat_ChannelSeparator.moduleOptions = {}

--[[------------------------------------------------
    Module Event Functions
------------------------------------------------]]--

-- things to do when the module is initialized
function Prat_ChannelSeparator:OnModuleInit()
    self.OriginalChatTypeGroup = {}
    self.OriginalChannelMenuChatTypeGroups = {}
end

-- things to do when the module is enabled
function Prat_ChannelSeparator:OnModuleEnable()
    self.OriginalChatTypeGroup = ChatTypeGroup
    ChatTypeGroup["GUILD"] = {
        "CHAT_MSG_GUILD",
        "GUILD_MOTD",
        "CHAT_MSG_OFFICER",
    };
    ChatTypeGroup["PARTY"] = {
        "CHAT_MSG_PARTY",
    };
    ChatTypeGroup["RAID"] = {
        "CHAT_MSG_RAID",
        "CHAT_MSG_RAID_LEADER",
        "CHAT_MSG_RAID_WARNING",
    };
    ChatTypeGroup["BATTLEGROUND"] = {
        "CHAT_MSG_BATTLEGROUND",
        "CHAT_MSG_BATTLEGROUND_LEADER",
    };
    self.OriginalChannelMenuChatTypeGroups = ChannelMenuChatTypeGroups
    ChannelMenuChatTypeGroups = {};
    ChannelMenuChatTypeGroups[1] = "SAY";
    ChannelMenuChatTypeGroups[2] = "YELL";
    ChannelMenuChatTypeGroups[3] = "GUILD";
    ChannelMenuChatTypeGroups[4] = "WHISPER";
    ChannelMenuChatTypeGroups[5] = "PARTY";
    ChannelMenuChatTypeGroups[6] = "RAID";
    ChannelMenuChatTypeGroups[7] = "BATTLEGROUND";
    local add = false
    for i=1,NUM_CHAT_WINDOWS do
        frame = getglobal("ChatFrame"..i)
        add = false
        local messageTypeList = frame.messageTypeList;
        if ( messageTypeList ) then
            for joinedIndex, joinedValue in pairs(messageTypeList) do
                if ( joinedValue == "PARTY" ) then
                    add = true
                end
            end
            if add then
                ChatFrame_AddMessageGroup(frame, "RAID")
                ChatFrame_AddMessageGroup(frame, "BATTLEGROUND")
            end
        end
    end
end

-- things to do when the module is disabled
function Prat_ChannelSeparator:OnModuleDisable()
    ChatTypeGroup = {}
    ChatTypeGroup = self.OriginalChatTypeGroup
    ChannelMenuChatTypeGroups = {}
    ChannelMenuChatTypeGroups = self.OriginalChannelMenuChatTypeGroups
end