﻿--[[
Name: Prat_History
Revision: $Revision: 24187 $
Author(s): Krtek (krtek4@gmail.com)
Website: http://www.wowace.com/files/index.php?path=Prat/
Documentation: http://www.wowace.com/wiki/Prat/Integrated_Modules#History
SVN: http://svn.wowace.com/wowace/trunk/Prat/
Description: Module for Prat that adds chat history options.
Dependencies: Prat
]]

local L = AceLibrary("AceLocale-2.2"):new("PratHistory")

L:RegisterTranslations("enUS", function() return {
    ["History"] = true,
    ["Chat history options."] = true,
    ["Set Lines"] = true,
    ["Set number of History lines for each window."] = true,
    ["Chat%dHistory"] = true,
    ["Sets history for chat window %d."] = true,
    ["Toggle"] = true,
    ["Toggle the module on and off."] = true,
} end)

L:RegisterTranslations("zhCN", function() return {
	["History"] = "历史",
	["Chat history options."] = "聊天历史选项。",
	["Set Lines"] = "设置",
	["Set number of History lines for each window."] = "为每一聊天窗口设置历史记录行数.",
    ["Chat%dHistory"] = "窗口%d历史",
    ["Sets history for chat window %d."] = "设置聊天窗口%d聊天历史记录",
	["Toggle"] = "切换",
	["Toggle the module on and off."] = "切换此模块的打开与关闭。",
} end)

L:RegisterTranslations("koKR", function() return {
    ["History"] = "이력",
    ["Chat history options."] = "대화 이력 설정입니다.",
    ["Set Lines"] = "라인수 설정",
    ["Set number of History lines for each window."] = "각각의 대화창에 대한 이력의 라인 수를 설정합니다.",
    ["Chat%dHistory"] = "대화창%d이력",
    ["Sets history for chat window %d."] = "대화창 %d에 대한 이력을 설정합니다.",
    ["Toggle"] = "사용",
    ["Toggle the module on and off."] = "기능 사용 여부를 결정합니다.",
} end)

L:RegisterTranslations("zhTW", function() return {
    ["History"] = "記錄",
    ["Chat history options."] = "聊天記錄選項。",
    ["Set Lines"] = "設定",
    ["Set number of History lines for each window."] = "為每一聊天視窗設定記錄行數。",
    ["Chat%dHistory"] = "視窗%d記錄",
    ["Sets history for chat window %d."] = "設定聊天視窗 %d 聊天記錄。",
    ["Toggle"] = "切換",
    ["Toggle the module on and off."] = "切換打開或關閉此模組。",
} end)

L:RegisterTranslations("esES", function() return {
    ["History"] = "Historial",
    ["Chat history options."] = "Opciones del historial de chat",
    ["Set Lines"] = "Establecer",
    ["Set number of History lines for each window."] = "Establece el n\195\186mero de l\195\173neas de historial para cada ventana",
    ["Chat%dHistory"] = "Historial del Chat %d",
    ["Sets history for chat window %d."] = "Establece el historial para la ventana de chat %d",
    ["Toggle"] = "Activar",
    ["Toggle the module on and off."] = "Activa y desactiva este m\195\179dulo.",
} end)

L:RegisterTranslations("deDE", function() return {
    ["History"] = "Verlauf",
    ["Chat history options."] = "Chat Verlauf Optionen.",
    ["Set Lines"] = "Setze",
    ["Set number of History lines for each window."] = "Setze die Anzahl der Verlauf Zeilen f\195\188r jedes Fenster.",
    ["Chat%dHistory"] = "Chat %d Verlauf",
    ["Sets history for chat window %d."] = "Setze Verlauf f\195\188r Chatfenster %d.",
    ["Toggle"] = "Einschalten",
    ["Toggle the module on and off."] = "Schaltet das Modul an und aus.",
} end)

Prat_History = Prat:NewModule(Prat:RequestModuleName("PratHistory"))

Prat_History.revision = tonumber(string.sub("$Revision: 24187 $", 12, -3))

Prat_History.defaultDB = {
    on = true,
    chatlines = {128, 128, 128, 128, 128, 128, 128, 128},
}

Prat_History.moduleOptions = {}

function Prat_History:GetModuleOptions()
    self.moduleOptions = {
        name = L["History"],
        desc = L["Chat history options."],
        type = "group",
        args = {
            set = {
                name = L["Set Lines"],
                desc = L["Set number of History lines for each window."],
                type = "group",
                args = {
                    chat1 = {
                        name = string.format(L["Chat%dHistory"], 1),
                        desc = string.format(L["Sets history for chat window %d."], 1),
                        type = "range",
                        min = 50,
                        max = 500,
                        step = 1,
                        get = function() return self.db.profile.chatlines[1] end,
                        set = function(v) Prat_History:SetHistory(1, v) end,
                    },
                    chat2 = {
                        name = string.format(L["Chat%dHistory"], 2),
                        desc = string.format(L["Sets history for chat window %d."], 2),
                        type = "range",
                        min = 50,
                        max = 500,
                        step = 1,
                        get = function() return self.db.profile.chatlines[2] end,
                        set = function(v) Prat_History:SetHistory(2, v) end,
                    },
                    chat3 = {
                        name = string.format(L["Chat%dHistory"], 3),
                        desc = string.format(L["Sets history for chat window %d."], 3),
                        type = "range",
                        min = 50,
                        max = 500,
                        step = 1,
                        get = function() return self.db.profile.chatlines[3] end,
                        set = function(v) Prat_History:SetHistory(3, v) end,
                    },
                    chat4 = {
                        name = string.format(L["Chat%dHistory"], 4),
                        desc = string.format(L["Sets history for chat window %d."], 4),
                        type = "range",
                        min = 50,
                        max = 500,
                        step = 1,
                        get = function() return self.db.profile.chatlines[4] end,
                        set = function(v) Prat_History:SetHistory(4, v) end,
                    },
                    chat5 = {
                        name = string.format(L["Chat%dHistory"], 5),
                        desc = string.format(L["Sets history for chat window %d."], 5),
                        type = "range",
                        min = 50,
                        max = 500,
                        step = 1,
                        get = function() return self.db.profile.chatlines[5] end,
                        set = function(v) Prat_History:SetHistory(5, v) end,
                    },
                    chat6 = {
                        name = string.format(L["Chat%dHistory"], 6),
                        desc = string.format(L["Sets history for chat window %d."], 6),
                        type = "range",
                        min = 50,
                        max = 500,
                        step = 1,
                        get = function() return self.db.profile.chatlines[6] end,
                        set = function(v) Prat_History:SetHistory(6, v) end,
                    },
                    chat7 = {
                        name = string.format(L["Chat%dHistory"],7),
                        desc = string.format(L["Sets history for chat window %d."], 7),
                        type = "range",
                        min = 50,
                        max = 500,
                        step = 1,
                        get = function() return self.db.profile.chatlines[7] end,
                        set = function(v) Prat_History:SetHistory(7, v) end,
                    },
                }
            },
            toggle = {
                name = L["Toggle"],
                desc = L["Toggle the module on and off."],
                type = "toggle",
                order = 200,
                get = function() return self.db.profile.on end,
                set = function(v) self.db.profile.on = Prat:ToggleModuleActive("PratHistory") end
            }
        }
    }
   	return self.moduleOptions
end

function Prat_History:OnModuleEnable()
    for i=1,NUM_CHAT_WINDOWS do
        self:SetHistory(i,self.db.profile.chatlines[i])
    end
end

function Prat_History:OnModuleDisable()
    for i=1,NUM_CHAT_WINDOWS do
        self:SetHistory(i,128)
    end
end

--[[------------------------------------------------
	Core Functions
------------------------------------------------]]--

function Prat_History:SetHistory(id, lines)
    if lines ~= self.db.profile.chatlines[id] then
    	getglobal("ChatFrame"..id):SetMaxLines(lines)
		self.db.profile.chatlines[id] = lines
	end
end