util = {}



function util:print(text, name, r, g, b, frame, delay)
	if not text or text:len() == 0 then
		text = " "
	end
	if not name or name == AceConsole then
	else
		text = "|cffffff78" .. tostring(name) .. ":|r " .. text
	end
	local last_color
	for t in text:gmatch("[^\n]+") do
		(frame or DEFAULT_CHAT_FRAME):AddMessage(last_color and "|cff" .. last_color .. t or t, r, g, b, nil, delay or 5)
		if not last_color or t:find("|r") or t:find("|c") then
			last_color = t:match(".*|c[fF][fF](%x%x%x%x%x%x)[^|]-$")
		end
	end
end

local real_tostring = tostring

function util:tostring(t)
	if type(t) == "table" then
		if type(rawget(t, 0)) == "userdata" and type(t.GetObjectType) == "function" then
			return ("<%s:%s>"):format(t:GetObjectType(), t:GetName() or "(anon)")
		end
	end
	return real_tostring(t)
end

local getkeystring

function util:isList(t)
	local n = #t
	for k,v in pairs(t) do
		if type(k) ~= "number" then
			return false
		elseif k < 1 or k > n then
			return false
		end
	end
	return true
end

local findGlobal = setmetatable({}, {__index=function(self, t)
	for k,v in pairs(_G) do
		if v == t then
			k = tostring(k)
			self[v] = k
			return k
		end
	end
	self[t] = false
	return false
end})


local recurse = {}
local timeToEnd
local GetTime = GetTime
local type = type
function util:literal_tostring_prime(t, depth, order_table)
	if type(t) == "string" then
		return ("|cff00ff00%q|r"):format((t:gsub("|", "||")))
	elseif type(t) == "table" then
		if t == _G then
			return "|cffffea00_G|r"
		end
		if type(rawget(t, 0)) == "userdata" and type(t.GetObjectType) == "function" then
			return ("|cffffea00<%s:%s>|r"):format(t:GetObjectType(), t:GetName() or "(anon)")
		end
	
		if recurse[t] then
			local g = findGlobal[t]
			if g then
				return ("|cff9f9f9f<Recursion _G[%q]>|r"):format(g)
			else
				return ("|cff9f9f9f<Recursion %s>|r"):format(real_tostring(t):gsub("|", "||"))
			end
		elseif GetTime() > timeToEnd then
			local g = findGlobal[t]
			if g then
				return ("|cff9f9f9f<Timeout _G[%q]>|r"):format(g)
			else
				return ("|cff9f9f9f<Timeout %s>|r"):format(real_tostring(t):gsub("|", "||"))
			end
		elseif depth >= 2 then
			local g = findGlobal[t]
			if g then
				return ("|cff9f9f9f<_G[%q]>|r"):format(g)
			else
				return ("|cff9f9f9f<%s>|r"):format(real_tostring(t):gsub("|", "||"))
			end
		end
		recurse[t] = true
		if next(t) == nil then
			return "{}"
		elseif next(t, (next(t))) == nil then
			local k, v = next(t)
			if k == 1 then
				return "{ " ..util:literal_tostring_prime(v, depth+1) .. " }"
			else
				return "{ " .. util:getkeystring(k, depth+1) .. " = " ..util:literal_tostring_prime(v, depth+1) .. " }"
			end
		end
		local s
		local g = findGlobal[t]
		if g then
			s = ("{ |cff9f9f9f-- _G[%q]|r\n"):format(g)
		else
			s = "{ |cff9f9f9f-- " .. real_tostring(t):gsub("|", "||") .. "|r\n"
		end
		if util:isList(t) then
			for i = 1, #t do
				s = s .. ("    "):rep(depth+1) ..util:literal_tostring_prime(t[i], depth+1) .. (i == #t and "\n" or ",\n")
			end
		else
		if order_table then
			local v
			for _,k in ipairs(order_table) do
				v = t[k]
				if v then 
					s = s .. ("    "):rep(depth+1) .. getkeystring(k, depth+1) .. " = " ..util:literal_tostring_prime(v, depth+1) .. (next(t, k) == nil and "\n" or ",\n")
				end
			end
		else
			for k,v in pairs(t) do
				s = s .. ("    "):rep(depth+1) .. getkeystring(k, depth+1) .. " = " ..util:literal_tostring_prime(v, depth+1) .. (next(t, k) == nil and "\n" or ",\n")
			end
		end

		end
		if g then
			s = s .. ("    "):rep(depth) .. string.format("} |cff9f9f9f-- _G[%q]|r", g)
		else
			s = s .. ("    "):rep(depth) .. "} |cff9f9f9f-- " .. real_tostring(t):gsub("|", "||")
		end
		return s
	end
	if type(t) == "number" then
		return "|cffff7fff" .. real_tostring(t) .. "|r"
	elseif type(t) == "boolean" then
		return "|cffff9100" .. real_tostring(t) .. "|r"
	elseif t == nil then
		return "|cffff7f7f" .. real_tostring(t) .. "|r"
	else
		return "|cffffea00" .. real_tostring(t) .. "|r"
	end
end

function getkeystring(t, depth)
	if type(t) == "string" then
		if t:find("^[%a_][%a%d_]*$") then
			return "|cff7fd5ff" .. t .. "|r"
		end
	end
	return "[" ..util:literal_tostring_prime(t, depth) .. "]"
end

local get_stringed_args
do
	local function g(value, ...)
		if select('#', ...) == 0 then
			return util:literal_tostring_prime(value, 1)
		end
		return util:literal_tostring_prime(value, 1) .. ", " .. g(...)
	end

	local function f(success, ...)
		if not success then
			return
		end
		return g(...)
	end

	function get_stringed_args(func, ...)
		return f(pcall(func, ...))
	end
end

function util:my_sort(alpha, bravo)
	if not alpha or not bravo then
		return false
	end
	return tostring(alpha):lower() < tostring(bravo):lower()
end


-- Copy Paste Reuse (How Modern)
function util:destroyTable(t)
	setmetatable(t, nil)
	for k,v in pairs(t) do
		t[k] = nil 
	end
end

function util:copyTable(src, dst)
	if dst == nil then dst = {} end
	for k,v in pairs(src) do 
		if type(v) ~="table" then
			dst[k] = v 
		else
			util:copyTable(v, dst[k]) 
		end 
	end
end

function util:orderedCopy(src, dst, order)
	util:copyTable(src, dst)
end

function util:tableMerge(src, dst, replace)
	if dst == nil then dst = {} end
	for k,v in pairs(src) do 
		if type(v) ~="table" then
			dst[k] = replace and dst[k] or v 
		else
			if replace or (not dst[k]) then
				util:copyTable(v, dst[k]) 
			end
		end 
	end
end
local tmp = {}
local tmp2 = {}
function util:literal_tostring_frame(t)
	local s = ("|cffffea00<%s:%s|r\n"):format(t:GetObjectType(), t:GetName() or "(anon)")
	local __index = getmetatable(t).__index
	for k in pairs(t) do
		if k ~= 0 then
			tmp2[k] = true
		end
	end
	for k in pairs(__index) do
		tmp2[k] = true
	end
	for k in pairs(tmp2) do
		tmp[#tmp+1] = k
		tmp2[k] = nil
	end
	table.sort(tmp, my_sort)
	local first = true
	for i,k in ipairs(tmp) do
		local v = t[k]
		local good = true
		if k == "GetPoint" then
			for i = 1, t:GetNumPoints() do
				if not first then
					s = s .. ",\n"
				else
					first = false
				end
				s = s .. "    " .. getkeystring(k, 1) .. "(" ..util:literal_tostring_prime(i, 1) .. ") => " .. get_stringed_args(v, t, i)
			end
		elseif type(v) == "function" and type(k) == "string" and (k:find("^Is") or k:find("^Get") or k:find("^Can")) then
			local q = get_stringed_args(v, t)
			if q then
				if not first then
					s = s .. ",\n"
				else
					first = false
				end
				s = s .. "    " .. getkeystring(k, 1) .. "() => " .. q
			end
		elseif type(v) ~= "function" then
			if not first then
				s = s .. ",\n"
			else
				first = false
			end
			s = s .. "    " .. getkeystring(k, 1) .. " = " ..util:literal_tostring_prime(v, 1)
		else
			good = false
		end
	end
	for k in pairs(tmp) do
		tmp[k] = nil
	end
	s = s .. "\n|cffffea00>|r"
	return s
end

function util:literal_tostring(t, only, order)
	timeToEnd = GetTime() + 0.2
	local s
	if only and type(t) == "table" and type(rawget(t, 0)) == "userdata" and type(t.GetObjectType) == "function" then
		s =util:literal_tostring_frame(t)
	else
		s =util:literal_tostring_prime(t, 0, order)
	end
	for k,v in pairs(recurse) do
		recurse[k] = nil
	end
	for k,v in pairs(findGlobal) do
		findGlobal[k] = nil
	end
	return s
end

function util:tostring_args(a1, ...)
	if select('#', ...) < 1 then
		return tostring(a1)
	end
	return tostring(a1), tostring_args(...)
end

function util:literal_tostring_args_ordered(order, a1, ...)
	if select('#', ...) < 1 then
		return util:literal_tostring(a1, order)
	end
	return util:literal_tostring(a1, order),util:literal_tostring_args(...)
end
function util:literal_tostring_args(a1, ...)
	if select('#', ...) < 1 then
		return util:literal_tostring(a1)
	end
	return util:literal_tostring(a1),util:literal_tostring_args(...)
end

util.keytable = nil
function util:setKeytable(t)
	for i,v in ipairs(t) do
		self.keytable[i] = v
	end
end

function util:sort_keytable(a, b)
	if not a or not b then
		return false
	end
	if not keytable[a] or not keytable[b] then
		return false
	end
	return keytable[a] > keytable[b]
end

-- Append any extra keys found in the table
-- to the end of the keys table
local mytmp = {}
function util:merge_keys(keys, t)
	if not t then return keys end
	
	local add = true
	for k,v in pairs(t) do
		add = true
		for _,key in ipairs(keys) do
			if key == k then
				add = false
			end
		end
		if add then table.insert(keys, k) end
	end
	return keys
end


CLR = {}

CLR.COLOR_NONE = nil

function CLR:GetHexColor(color)
    return string.format("%02x%02x%02x", color.r*255, color.g*255, color.b*255)
end

function CLR:Colorize(hexColor, text)
    if text == nil then text = "" end

    if hexColor == CLR.COLOR_NONE then
        return text
    end
    
    return "|cff" .. tostring(hexColor or 'ffffff') .. tostring(text) .. "|r"
end


